# import libs

# SECTION: Joback method settings
JOBACK_DATA_FILE = 'joback.csv'

# NOTE: tables
JOBACK_TABLE_COLUMN_GROUP = 'Group'

# SECTION: zabransky-ruzicka method settings
ZABRANSKY_RUZICKA_DATA_FILE_1 = 'zabransky-ruzicka-1.csv'
ZABRANSKY_RUZICKA_DATA_FILE_2 = 'zabransky-ruzicka-2.csv'

# NOTE: tables
ZABRANSKY_RUZICKA_TABLE_COLUMN_GROUP = 'Group'

# NOTE: reference
ZABRANSKY_RUZICKA_REFERENCE = """Zábranský, Milan, and Vlastimil Růžička Jr. "Estimation of the heat capacities of organic liquids as a function of temperature using group additivity: an amendment." Journal of physical and chemical reference data 33.4 (2004): 1071-1081.
"""
